@@ -3,53 +3,84 @@ use 5.010001;
 use strict;
 use warnings;
 
-our $VERSION = '0.08';
+use Carp ();
+
+our $VERSION = '0.09';
 our $PREFIX  = '__cag_';
 
 require XSLoader;
 XSLoader::load('Class::Accessor::Inherited::XS', $VERSION);
 
-{
-    no strict 'refs';
-    
-    sub import {
-        my $pkg = shift;
-        return unless scalar @_;
+my $REGISTERED_TYPES = {};
+register_type(inherited => {no_cb => 1});
 
-        my $class = caller;
-        my %opts = ref($_[0]) eq 'HASH' ? %{ $_[0] } : @_;
+sub import {
+    my $pkg = shift;
+    return unless scalar @_;
 
-        if (my $inherited = $opts{inherited}) {
-            if (ref($inherited) eq 'HASH') {
-                mk_inherited_accessor($class, $_, $inherited->{$_}) for keys %$inherited;
+    my $class = caller;
+    my %opts = ref($_[0]) eq 'HASH' ? %{ $_[0] } : @_;
 
-            } elsif (ref($inherited) eq 'ARRAY') {
-                mk_inherited_accessor($class, $_, $_) for @$inherited;
+    for my $type (keys %opts) {
+        my $type_info = $REGISTERED_TYPES->{$type};
 
-            } else {
-                warn "Can't understand format for inherited accessors initializer for class $class";
-            }
+        if (!defined $type_info) {
+            Carp::confess("Don't know how to install '$type' accessors");
+        }
+
+        my $accessors = $opts{$type};
+        if (ref($accessors) eq 'HASH') {
+            $type_info->{installer}->($class, $_, $accessors->{$_}) for keys %$accessors;
+
+        } elsif (ref($accessors) eq 'ARRAY') {
+            $type_info->{installer}->($class, $_, $_) for @$accessors;
+
+        } else {
+            Carp::confess("Can't understand format for '$type' accessors initializer");
         }
     }
+}
 
-    sub mk_inherited_accessors {
-        my $class = shift;
+sub mk_inherited_accessors {
+    my $class = shift;
 
-        for my $entry (@_) {
-            if (ref($entry) eq 'ARRAY') {
-                mk_inherited_accessor($class, @$entry);
-            } else {
-                mk_inherited_accessor($class, $entry, $entry);
-            }
+    for my $entry (@_) {
+        if (ref($entry) eq 'ARRAY') {
+            _mk_inherited_accessor($class, @$entry);
+        } else {
+            _mk_inherited_accessor($class, $entry, $entry);
         }
     }
+}
 
-    #this function is NOT part of the public API
-    sub mk_inherited_accessor {
-        my($class, $name, $field) = @_;
+sub register_type {
+    my ($type, $args) = @_;
 
-        Class::Accessor::Inherited::XS::install_inherited_accessor("${class}::${name}", $field, $PREFIX.$field);
+    if (exists $REGISTERED_TYPES->{$type}) {
+        Carp::confess("Type '$type' has already been registered");
     }
+
+    if (exists $args->{no_cb}) {
+        $args->{installer} = \&_mk_inherited_accessor;
+
+    } else {
+        $args->{installer} = sub {
+            my ($class, $name, $field) = @_;
+            install_inherited_cb_accessor("${class}::${name}", $field, $PREFIX.$field, $args->{read_cb}, $args->{write_cb});
+        };
+    }
+
+    $REGISTERED_TYPES->{$type} = $args;
+}
+
+#
+#   Functions below are NOT part of the public API
+#
+
+sub _mk_inherited_accessor {
+    my ($class, $name, $field) = @_;
+
+    install_inherited_accessor("${class}::${name}", $field, $PREFIX.$field);
 }
 
 1;
@@ -57,7 +88,7 @@ __END__
 
 =head1 NAME
 
-Class::Accessor::Inherited::XS - fast XS inherited accessors
+Class::Accessor::Inherited::XS - Fast XS inherited accessors
 
 =head1 SYNOPSIS
 
@@ -79,13 +110,14 @@ Class::Accessor::Inherited::XS - fast XS inherited accessors
 
 =head1 DESCRIPTION
 
-This module provides very fast implementation for 'inherited' accessors, that were introduced 
-by the L<Class::Accessor::Grouped> module. They give you capability to override values set in 
+This module provides a very fast implementation for 'inherited' accessors, that were introduced
+by the L<Class::Accessor::Grouped> module. They give you a capability to override values set in
 a parent class with values set in childs or object instances. Generated accessors are compatible with
 L<Class::Accessor::Grouped> generated ones.
 
 Since this module focuses primary on speed, it provides no capability to have your own per-class
-getters/setters logic (like overriding L<get_inherited>/L<set_inherited> in L<Class::Accessor::Grouped>).
+getters/setters logic (like overriding L<get_inherited>/L<set_inherited> in L<Class::Accessor::Grouped>),
+but it gives you an ability to register a single get/set callback for you own accessor types.
 
 =head1 UTF-8
 
@@ -119,9 +151,35 @@ Here are results from a benchmark run on perl 5.20.1 (see bench folder):
   obj_caix          18642286/s           7751%   1740%   1539%       1349%              522%     117%         109%     11%       --       -25%
   obj_direct        24807382/s          10348%   2349%   2081%       1829%              728%     188%         178%     48%      33%         --
 
+=head1 EXTENDING
+
+    package MyAccessor;
+    # 'register_type' isn't exported
+    Class::Accessor::Inherited::XS::register_type(
+        inherited_cb => {on_read => sub {}, on_write => sub{}},
+    );
+
+    package MyClass;
+    use MyAccessor;
+    use Class::Accessor::Inherited::XS {
+        inherited    => ['foo'],
+        inherited_cb => ['bar'],
+    };
+
+You can register new inherited accessor types with associated read/write callbacks. They're still
+'inherited', but you can modify return values. Those new types can be installed using
+the L<Class::Accessor::Inherited::XS> import() call. Unlike L<Class::Accessor::Grouped>,
+here's a single callback per accessor type, without any inheritance lookups for get_*/set_* functions.
+
+B<on_read> callback gets a single argument - from a normal 'inherited' accessor. It's return value is a new
+accessor's return value (and is not stored anywhere).
+
+B<on_write> callback gets two arguments - original args from the accessor's call. It's return value is saved
+instead of the user's supplied one. Exceptions thrown from this callback will cancel store.
+
 =head1 PROFILING WITH Devel::NYTProf
 
-To perform it's task, L<Devel::NYTProf> hooks into perl interpreter by replacing default behaviour for calling subroutines
+To perform it's task, L<Devel::NYTProf> hooks into the perl interpreter by replacing default behaviour for calling subroutines
 on the opcode level. To squeeze last bits of performance, L<Class::Accessor::Inherited::XS> does the same, but separately
 on each call site of its accessors. It turns out into CAIX favor - L<Devel::NYTProf> sees only first call to CAIX
 accessor, but all subsequent ones become invisible to the subs profiler.
